@@ -1,4 +1,4 @@
-# Copyright 2008-2018 Gentoo Foundation
+# Copyright 2008-2020 Gentoo Authors
 # Distributed under the terms of the GNU General Public License v2
 
 import fcntl
@@ -9,6 +9,7 @@ import sys
 import portage
 from portage import os, _encodings, _unicode_encode
 from portage.util.futures import asyncio
+from portage.util.futures._asyncio.streams import _writer
 from portage.util.futures.compat_coroutine import coroutine
 from portage.util.futures.unix_events import _set_nonblocking
 from _emerge.AbstractPollTask import AbstractPollTask
@@ -24,12 +25,13 @@ class PipeLogger(AbstractPollTask):
 	"""
 
 	__slots__ = ("input_fd", "log_file_path", "stdout_fd") + \
-		("_io_loop_task", "_log_file", "_log_file_real")
+		("_io_loop_task", "_log_file", "_log_file_nb", "_log_file_real")
 
 	def _start(self):
 
 		log_file_path = self.log_file_path
 		if hasattr(log_file_path, 'write'):
+			self._log_file_nb = True
 			self._log_file = log_file_path
 			_set_nonblocking(self._log_file.fileno())
 		elif log_file_path is not None:
@@ -45,9 +47,9 @@ class PipeLogger(AbstractPollTask):
 				mode=0o660)
 
 		if isinstance(self.input_fd, int):
-			fd = self.input_fd
-		else:
-			fd = self.input_fd.fileno()
+			self.input_fd = os.fdopen(self.input_fd, 'rb', 0)
+
+		fd = self.input_fd.fileno()
 
 		fcntl.fcntl(fd, fcntl.F_SETFL,
 			fcntl.fcntl(fd, fcntl.F_GETFL) | os.O_NONBLOCK)
@@ -62,7 +64,7 @@ class PipeLogger(AbstractPollTask):
 				fcntl.fcntl(fd, fcntl.F_SETFD,
 					fcntl.fcntl(fd, fcntl.F_GETFD) | fcntl.FD_CLOEXEC)
 
-		self._io_loop_task = asyncio.ensure_future(self._io_loop(fd), loop=self.scheduler)
+		self._io_loop_task = asyncio.ensure_future(self._io_loop(self.input_fd), loop=self.scheduler)
 		self._io_loop_task.add_done_callback(self._io_loop_done)
 		self._registered = True
 
@@ -72,10 +74,11 @@ class PipeLogger(AbstractPollTask):
 			self.returncode = self._cancelled_returncode
 
 	@coroutine
-	def _io_loop(self, fd):
+	def _io_loop(self, input_file):
 		background = self.background
 		stdout_fd = self.stdout_fd
 		log_file = self._log_file 
+		fd = input_file.fileno()
 
 		while True:
 			buf = self._read_buf(fd)
@@ -87,9 +90,14 @@ class PipeLogger(AbstractPollTask):
 				try:
 					yield future
 				finally:
+					# The loop and input file may have been closed.
 					if not self.scheduler.is_closed():
-						self.scheduler.remove_reader(fd)
 						future.done() or future.cancel()
+						# Do not call remove_reader in cases where fd has
+						# been closed and then re-allocated to a concurrent
+						# coroutine as in bug 716636.
+						if not input_file.closed:
+							self.scheduler.remove_reader(fd)
 				continue
 
 			if not buf:
@@ -131,30 +139,16 @@ class PipeLogger(AbstractPollTask):
 								fcntl.F_GETFL) ^ os.O_NONBLOCK)
 
 				if log_file is not None:
-					if isinstance(log_file, gzip.GzipFile):
-						# Use log_file.write since data written directly
-						# to the file descriptor bypasses compression.
+					if self._log_file_nb:
+						# Use the _writer function which uses os.write, since the
+						# log_file.write method looses data when an EAGAIN occurs.
+						yield _writer(log_file, buf, loop=self.scheduler)
+					else:
+						# For gzip.GzipFile instances, the above _writer function
+						# will not work because data written directly to the file
+						# descriptor bypasses compression.
 						log_file.write(buf)
 						log_file.flush()
-						continue
-
-					write_buf = buf
-					while write_buf:
-						try:
-							# Use os.write, since the log_file.write method
-							# looses data when an EAGAIN occurs.
-							write_buf = write_buf[os.write(log_file.fileno(), write_buf):]
-						except EnvironmentError as e:
-							if e.errno != errno.EAGAIN:
-								raise
-							future = self.scheduler.create_future()
-							self.scheduler.add_writer(self._log_file.fileno(), future.set_result, None)
-							try:
-								yield future
-							finally:
-								if not self.scheduler.is_closed():
-									self.scheduler.remove_writer(self._log_file.fileno())
-									future.done() or future.cancel()
 
 	def _io_loop_done(self, future):
 		try:
@@ -168,15 +162,15 @@ class PipeLogger(AbstractPollTask):
 	def _unregister(self):
 		if self.input_fd is not None:
 			if isinstance(self.input_fd, int):
-				self.scheduler.remove_reader(self.input_fd)
 				os.close(self.input_fd)
-			else:
+			elif not self.input_fd.closed:
 				self.scheduler.remove_reader(self.input_fd.fileno())
 				self.input_fd.close()
 			self.input_fd = None
 
 		if self._io_loop_task is not None:
-			self._io_loop_task.done() or self._io_loop_task.cancel()
+			if not self.scheduler.is_closed():
+				self._io_loop_task.done() or self._io_loop_task.cancel()
 			self._io_loop_task = None
 
 		if self.stdout_fd is not None:
@@ -184,8 +178,9 @@ class PipeLogger(AbstractPollTask):
 			self.stdout_fd = None
 
 		if self._log_file is not None:
-			self.scheduler.remove_writer(self._log_file.fileno())
-			self._log_file.close()
+			if not self._log_file.closed:
+				self.scheduler.remove_writer(self._log_file.fileno())
+				self._log_file.close()
 			self._log_file = None
 
 		if self._log_file_real is not None:
